package com.emato.ccnet.wx.manager.redis.jedis.cluster.mgt;

import com.emato.ccnet.wx.util.SerializeUtils;
import com.emato.ccnet.wx.util.SpringContextSupport;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import redis.clients.jedis.JedisCluster;
import redis.clients.util.SafeEncoder;

import java.util.Map;

/**
 * JedisCluster的管理类, 提供封装了JedisCluster操作Redis方法
 * 同时提供JedisCluster实例, 以提供JedisCluster原生操作Redis方法
 * @author Scott Chen
 * @since 1.0
 * 2018-01-07 18:42
 */
public class JedisClusterManager {
    private static final Logger logger = LoggerFactory.getLogger(JedisClusterManager.class);

    private static final ThreadLocal<JedisCluster> threadLocalJedisCluster = new ThreadLocal<>();
    private static final ThreadLocal<JedisClusterManager> threadLocalJedisClusterManager = new ThreadLocal<>();

    private JedisClusterManager() {}

    /**
     * 用于JedisCluster实例管理
     * @return
     */
    public static JedisClusterManager instance() {
        JedisClusterManager jedisClusterManager = threadLocalJedisClusterManager.get();
        if (jedisClusterManager == null) {
            jedisClusterManager = new JedisClusterManager();
            threadLocalJedisClusterManager.set(jedisClusterManager);
        }
        return jedisClusterManager;
    }

    /**
     * 直接提供JedisCluster实例, 可直接使用原生JedisCluster中方法
     * @return
     */
    public static JedisCluster jedisClusterInstance() {
        JedisCluster jedisCluster = threadLocalJedisCluster.get();
        if (jedisCluster == null) {
            jedisCluster = SpringContextSupport.getBean("jedisCluster");
            threadLocalJedisCluster.set(jedisCluster);
        }
        return jedisCluster;
    }


    //-------------------- 封装JedisCluster操作Redis Cluster方法 --------------------


    //-------------------- Binary --------------------

    /**
     * 字符序列化存储
     * @param key
     * @param value
     */
    public void setAsBinary(String key, String value) {
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);
            byte[] byteValue = SafeEncoder.encode(value);

            jedisCluster = jedisClusterInstance();
            jedisCluster.set(byteKey, byteValue);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set string as binary to redis error: " + e);
        }
    }

    /**
     * 自定义过期时间字符序列化存储
     * @param key
     * @param value
     * @param expire
     */
    public void setAsBinaryExpire(String key, String value, long expire) {
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);
            byte[] byteValue = SafeEncoder.encode(value);

            jedisCluster = jedisClusterInstance();
            jedisCluster.set(byteKey, byteValue);
            if (expire > 0) {
                jedisCluster.pexpire(byteKey, expire);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set string with expire as binary to redis error: " + e);
        }
    }

    /**
     * 获取序列化数据
     * @param key
     * @return
     */
    public String getFromBinary(String key) {
        String value = null;
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);

            jedisCluster = jedisClusterInstance();
            byte[] byteValue = jedisCluster.get(byteKey);

            value = (String) SerializeUtils.deserialize(byteValue);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("get string from binary in redis error: " + e);
        }
        return value;
    }

    /**
     * 自定义过期时间Map<String, String>序列化存储
     * Jedis在底层序列化Map<String, String>后再存储
     * @param key
     * @param hash
     * @param expire
     */
    public void hmsetAsBinaryExpire(final String key, final Map<String, String> hash, long expire) {
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);

            jedisCluster = jedisClusterInstance();
            jedisCluster.hmset(key, hash);
            if (expire > 0) {
                jedisCluster.pexpire(byteKey, expire);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set Map<String, String> with expire as binary in redis error: " + e);
        }
    }

    /**
     * 获取序列化数据
     * @param key
     * @return
     */
    public Map<String, String> hgetAllFromBinary(String key) {
        Map<String, String> value = null;
        JedisCluster jedisCluster;
        try {
            jedisCluster = jedisClusterInstance();
            value = jedisCluster.hgetAll(key);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("get string from binary in redis error: " + e);
        }
        return value;
    }

    /**
     * Map<String, Object>序列化存储
     * 自定义序列化Map<String, Object>后再存储
     * @param key
     * @param hash
     */
    public void hmsetObjectAsBinary(final String key, final Map<String, Object> hash) {
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);
            byte[] byteValue = SerializeUtils.serialize(hash);

            jedisCluster = jedisClusterInstance();
            jedisCluster.set(byteKey, byteValue);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set Map<String, Object> as binary to redis error: " + e);
        }
    }

    /**
     * 自定义过期时间Map<String, Object>序列化存储
     * 自定义序列化Map<String, Object>后再存储
     * @param key
     * @param hash
     * @param expire
     */
    public void hmsetObjectAsBinaryExpire(final String key, final Map<String, Object> hash, long expire) {
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);
            byte[] byteValue = SerializeUtils.serialize(hash);

            jedisCluster = jedisClusterInstance();
            jedisCluster.set(byteKey, byteValue);
            if (expire > 0) {
                jedisCluster.pexpire(byteKey, expire);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set Map<String, Object> with expire as binary to redis error: " + e);
        }
    }

    /**
     * 获取序列化数据
     * @param key
     * @return
     */
    public Map<String, Object> hgetAllObjectFromBinary(String key) {
        Map<String, Object> value = null;
        JedisCluster jedisCluster;
        try {
            byte[] byteKey = SafeEncoder.encode(key);

            jedisCluster = jedisClusterInstance();
            byte[] byteValue = jedisCluster.get(byteKey);

            value = (Map<String, Object>) SerializeUtils.deserialize(byteValue);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("get string from binary in redis error: " + e);
        }
        return value;
    }

    /**
     * 是否存在对应序列化key
     * @param key
     * @return
     */
    public boolean existsKeyFromBinary(String key) {
        JedisCluster jedisCluster;
        try {
            jedisCluster = jedisClusterInstance();
            byte[] byteKey = SafeEncoder.encode(key);
            return jedisCluster.exists(byteKey);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("exists key【" + key + "】 from binary in redis error: " + e);
        }
        return false;
    }



    //-------------------- String --------------------


    /**
     * 自定义过期时间字符串存储
     * @param key
     * @param value
     * @param expire
     */
    public void setExpire(String key, String value, long expire) {
        JedisCluster jedisCluster;
        try {
            jedisCluster = jedisClusterInstance();
            jedisCluster.set(key, value);
            if (expire > 0) {
                jedisCluster.pexpire(key, expire);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set string with expire to redis error: " + e);
        }
    }

    /**
     * 获取字符串类型
     * @param key
     */
    public String get(String key) {
        String value = null;
        JedisCluster jedisCluster;
        try {
            jedisCluster = jedisClusterInstance();
            value = jedisCluster.get(key);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("get string from redis error: " + e);
        }
        return value;
    }


    /**
     * 是否存在key
     * @param key
     * @return
     */
    public boolean existsKey(String key) {
        JedisCluster jedisCluster;
        try {
            jedisCluster = jedisClusterInstance();
            return jedisCluster.exists(key);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("exists key【" + key + "】 in redis error: " + e);
        }
        return false;
    }


    //-------------------- hash field  --------------------

    /**
     * hash field
     * @param key
     * @param field
     * @param value
     * @param expire
     */
    public void hset(final String key, final String field, final String value, int expire) {
        JedisCluster jedisCluster;
        try {
            jedisCluster = jedisClusterInstance();
            jedisCluster.hset(key, field, value);
            if (expire > 0) {
                jedisCluster.expire(key, expire);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("hset key【" + key + "】,field【" + field + "】 with expire to redis error: " + e);
        }
    }


    //-------------------- Object --------------------


    /**
     * 设置对象类型Object数据
     *
     * @param objectKey
     * @param objectValue
     */
    public void setObject(Object objectKey, Object objectValue) {
        JedisCluster jedisCluster;
        try {
            byte[] key = SerializeUtils.serialize(objectKey);
            byte[] value = SerializeUtils.serialize(objectValue);

            jedisCluster = jedisClusterInstance();
            jedisCluster.set(key, value);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set object with expire to redis error: " + e);
        }
    }

    /**
     * 自定义过期时间设置对象类型数据Object
     *
     * @param objectKey
     * @param objectValue
     * @param expire
     *            单位秒
     */
    public void setObjectExpire(Object objectKey, Object objectValue, int expire) {
        JedisCluster jedisCluster;
        try {
            byte[] key = SerializeUtils.serialize(objectKey);
            byte[] value = SerializeUtils.serialize(objectValue);

            jedisCluster = jedisClusterInstance();
            jedisCluster.set(key, value);
            if (expire > 0) {
                jedisCluster.expire(key, expire);
            }

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("set object with expire to redis error: " + e);
        }
    }

    /**
     * 获取Object数据
     *
     * @param objectKey
     * @return
     */
    public Object getObject(Object objectKey) {
        byte[] value = null;
        JedisCluster jedisCluster;
        try {
            byte[] key = SerializeUtils.serialize(objectKey);

            jedisCluster = jedisClusterInstance();
            value = jedisCluster.get(key);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("get object from redis error: " + e);
        }
        if (value == null) {
            return null;
        }
        return SerializeUtils.deserialize(value);
    }


}
